/*
 *  2016 Maciej Wiecierzewski
 */

var http = require('http');

var database = require("./database");
database = database.database;

var server = {
    server: null,
    port: 8080,
    databaseReady: false,
    
    init: function()
    {
        this.server = http.createServer();
        this.server.on('request', function(request, response) { this.onRequest(request, response); }.bind(this) );
        this.server.listen(this.port);
        
        console.log("Server listening on "+this.port);
    },
    
    onRequest: function(request, response)
    {
        var body = [];
        
        var onRequestData = function(chunk)
        {
            body.push(chunk);
        };

        var onRequestEnd = function()
        {
            body = Buffer.concat(body).toString();
            
            var requestBody = {
                    headers: request.headers,
                    method: request.method,
                    url: request.url,
                    body: body
                };
            
            this.sendResponse(requestBody, response);
        }.bind(this);

        var onRequestError = function(err)
        {
            console.log(err);
        };
        
        request.on('end', onRequestEnd);
        request.on('data', onRequestData);
        request.on('error', onRequestError);
    },
    
    sendResponse : function(requestBody, response)
    {
        var onResponseError = function(err)
        {
            console.log(err);
        };
        
        var onResponseReady = function(responseObj)
        {
            console.log("response ready");
            
            response.statusCode = 200;
            response.setHeader('Content-Type', 'application/json');
            
            var responseStr = JSON.stringify(responseObj);
            response.write(responseStr);

            response.end();
        };
        
        response.on('error', onResponseError);
        response.on('responseReady', onResponseReady);
        
        var url = requestBody.url.split("?");
        var get = [];
        
        if(url[1] !== undefined)
        {
            var arr = url[1].split("&");
            
            for(var i = 0; i < arr.length; i++)
            {
                var str = arr[i].split("=");
                get[str[0]] = str[1];
            }
        }
        
        var path = url[0].split("/");
        
        var action = path[path.length-1];
        
        if(action === 'get_words')
        {
            var selectCallback = function(error, result, fields)
            {
                var wordsArr = [];
                
                response.emit('responseReady', { result: "ok", words: result });
            }

            database.select( { get: ["id", "word"] }, selectCallback);
        }
        else if(action === 'insert')
        {
            var insertCallback = function(error, result, fields)
            {
                response.emit('responseReady', { result: "ok" });
            }
            
            var word = get["word"];

            database.insert(word, Date.now(), insertCallback);
        }
        else if(action === 'delete')
        {
            var deleteCallback = function(error, result, fields)
            {
                response.emit('responseReady', { result: "ok" });
            }
            
            var word = get["word"];

            database.delete(word, deleteCallback);
        }
        else
        {
            console.log("Url '"+requestBody.url+"' not found");
            response.emit('responseReady', { result: "error" });
        }
        
    }
};

module.exports.server = server;
